"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var system_1 = require("./system");
var Channel = /** @class */ (function () {
    function Channel(name, gusher) {
        this.name = name;
        this.gusher = gusher;
        this.subscribed = false;
        this.emitter = new events_1.EventEmitter();
    }
    Channel.prototype.trigger = function (event, data) {
        this.gusher.send(event, data, this.name);
    };
    Channel.prototype.bind = function (event, callback) {
        this.emitter.on(event, callback);
        return this;
    };
    Channel.prototype.unbind = function (event, callback) {
        this.emitter.removeListener(event, callback);
        return this;
    };
    Channel.prototype.unsubscribe = function () {
        this.gusher.send(system_1.Action.UNSUBSCRIBE, { channel: this.name });
    };
    Channel.prototype.handleEvent = function (event, data) {
        if (event === system_1.Event.SUBSCRIBE_SUCCESS ||
            event === system_1.Event.MULTI_SUBSCRIBE_SUCCESS) {
            this.subscribed = true;
        }
        this.emitter.emit(event, data);
    };
    Channel.prototype.subscribe = function () {
        this.gusher.send(system_1.Action.SUBSCRIBE, { channel: this.name });
    };
    Channel.prototype.disconnect = function () {
        this.subscribed = true;
    };
    return Channel;
}());
exports.Channel = Channel;
