"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var events_1 = require("events");
var logger_1 = __importDefault(require("./logger"));
var State;
(function (State) {
    State["INIT"] = "initialized";
    State["ERROR"] = "error";
    State["CLOSED"] = "closed";
    State["CONNECTING"] = "connecting";
    State["OPEN"] = "open";
})(State = exports.State || (exports.State = {}));
var EmitterEvent;
(function (EmitterEvent) {
    EmitterEvent["INIT"] = "initialized";
    EmitterEvent["ERROR"] = "error";
    EmitterEvent["CLOSED"] = "closed";
    EmitterEvent["CONNECTING"] = "connecting";
    EmitterEvent["OPEN"] = "open";
    EmitterEvent["MSG"] = "message";
})(EmitterEvent = exports.EmitterEvent || (exports.EmitterEvent = {}));
var Connection = /** @class */ (function () {
    function Connection(options) {
        this.url = "";
        this.token = "";
        this.url = options.url;
        this.token = options.token;
        this.state = State.INIT;
        this.emitter = new events_1.EventEmitter();
    }
    Connection.prototype.bind = function (event, callback) {
        this.emitter.on(event, callback);
        return this;
    };
    Connection.prototype.unbind = function (event, callback) {
        this.emitter.removeListener(event, callback);
        return this;
    };
    Connection.prototype.connect = function (token) {
        if (this.socket) {
            this.close();
        }
        var url = this.url || "ws://127.0.0.1";
        if (token) {
            url = url + "?token=" + token;
        }
        try {
            this.socket = new WebSocket(url);
        }
        catch (e) {
            this.onError(e);
            return false;
        }
        this.bindListeners();
        logger_1.default.log("Connecting", { url: this.url, token: this.token });
        this.changeState(State.CONNECTING);
        return true;
    };
    Connection.prototype.close = function () {
        if (this.socket) {
            this.socket.close();
            return true;
        }
        return false;
    };
    Connection.prototype.changeState = function (state, params) {
        this.state = state;
        this.emitter.emit(state, params);
    };
    Connection.prototype.bindListeners = function () {
        var _this = this;
        var socket = this.socket;
        socket.onopen = function (evt) {
            _this.onOpen(evt);
        };
        socket.onerror = function (evt) {
            _this.onError(evt);
        };
        socket.onclose = function (evt) {
            _this.onClose(evt);
        };
        socket.onmessage = function (message) {
            _this.onMessage(message);
        };
    };
    Connection.prototype.unbindListeners = function () {
        if (this.socket) {
            this.socket.onopen = null;
            this.socket.onerror = null;
            this.socket.onclose = null;
            this.socket.onmessage = null;
        }
    };
    Connection.prototype.onOpen = function (evt) {
        this.changeState(State.OPEN);
        if (this.socket) {
            this.socket.onopen = null;
        }
    };
    Connection.prototype.onError = function (error) {
        this.emitter.emit(EmitterEvent.ERROR, error);
    };
    Connection.prototype.onClose = function (closeEvent) {
        if (closeEvent) {
            this.changeState(State.CLOSED, {
                code: closeEvent.code,
                reason: closeEvent.reason,
                wasClean: closeEvent.wasClean,
            });
        }
        else {
            this.changeState(State.CLOSED);
        }
        this.unbindListeners();
        this.socket = undefined;
    };
    Connection.prototype.onMessage = function (event) {
        var message;
        try {
            message = JSON.parse(event.data);
        }
        catch (err) {
            logger_1.default.log({ error: err });
            message = {
                event: "",
                data: "",
            };
        }
        logger_1.default.log("Event recd", message);
        this.emitter.emit(EmitterEvent.MSG, message);
    };
    Connection.prototype.send = function (event, data, channel) {
        var message = { event: event, data: data };
        if (channel) {
            message.channel = channel;
        }
        logger_1.default.log("Event sent", message);
        if (this.socket) {
            this.socket.send(JSON.stringify(message));
        }
    };
    return Connection;
}());
exports.Connection = Connection;
